﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Reflection;
using System.IO;

//the InventorRegistration namespace contains the utility classes that Inventor provides via the InentorRegUtils assembly
//(you would need to add a reference to the InventorRegUtils assembly that will be installed and registered when Inventor is installed)
using InventorRegistration;

//The intent of this re-registration executable is to provide a means for controlling the versions of Inventor into 
//which the AddIn corresponding to this re-registration executable needs to be loaded. Typically, specifiying the 
//"SupportedSoftwareVersionEqualTo = x.." (where x corresponds to the version of Inventor that the AddIn needs to be 
//loaded into) and rev'ing the GUID corresponding to the AddIn's CLSID should avoid the problems that are associated 
//with having multiple versions of Inventor all of which load the corresponding version of the AddIn. (For example,
//if the user has installed Inventor 9 and Inventor 10 and the corresponding versions of the AddIns, let's say AddIn 9
//and AddIn 10 which have the same GUID for the CLSID, then, un-installing AddIn 9 will also un-install AddIn 10. If
//you want to preserve the AddIn's CLSID across multiple AddIn versions, then you need to implement a re-registration
//executable like this one to ensure that Inventor will be able to load the appropriate versions of the AddIns. 

//Th re-registration executable can specify that .NET assemblies as well as standard dll's need to be registered. 
//If needed, you can also add registry settings directly.

//The steps that need to be followed during installation of the AddIn application are:
//1) Install the re-registration executable (in this case, Registration.exe) along with the AddIn installation. 
//2) Invoke the re-registration executable with a "/install" command line argument during install. 
//	 This will ensure that the re-registration information is added to the registry the first time, Inventor will
//   then be able to find the path and run this executable if the AddIn needs to be loaded in the version of Inventor
//   that is currently being registered.
//3) Upon uninstall invoke the re-registration executable with a "/uninstall" argument before removing it from the disk. 
//	 This will ensure that the re-registration information is removed from the registry and Inventor will
//   not register and load the AddIn the next time Inventor goes through the registration process.

//Note: This re-registration executable will have to create a new GUID for each version of the executable that is 
//generated (typically, for each version of the AddIn) and also specify the versions of Inventor that the AddIn needs
//to be loaded in. The settings are specified in a class that implements the IAddInRegistrationInfo interface.
//whose methods Inventor will call if it finds the GUID corresponding to this re-registration executable in the registry 
//(HKEY_CLASSES_ROOT). 

//The re-registration executables are recognized by Inventor if if finds the Implemented Categories\{E956B1CC-1AA4-41c6-A40C-687B4A4AE0E9}
//under the GUID corresponding to this re-registration executable.

//Inventor will only know to call the properties and methods implemented by the IAddInRegistrationInfo interface, hence,
//this interface has to be implemented if you want to pass registration information to Inventor which will then take care
//of the actual registration.
 
class SimpleAddInRegInfo : IAddInRegistrationInfo
{
	//PROPERTIES: The following properties will be added to the registry by Inventor to identify the re-registration executable
	//These properties will be added when the executable is called with "/install" command line (which is typically done during
	//the installation of the AddIn)

	//re-registration GUID, create a new GUID for each version of the re-registration executable,
	//each version of the re-registration executable is typically associated with a corresponding version of the AddIn
	//application that needs to be registered with a particular version(s) of Inventor that the AddIn intends to be loaded in
    string m_guidStr = "{526C4A2C-6371-44de-9021-D29B9469D54C}";
	public string GuidString { get{return m_guidStr;}}

	//descriptive name for the re-registration executable
	string m_descriptiveName = "RegisterSimpleAddIn";
	public string DescriptiveName { get{return m_descriptiveName;}}

	//the full path name for this re-registration executable
	//Inventor will then be able to find and run this re-registration executable the next time it goes through the registry
	string m_executablePath = Assembly.GetExecutingAssembly().Location;
	public string ExecutablePath { get{return m_executablePath;}}

	//the version rules (e.g. "SupportedSoftwareVersionEqualTo"), the re-registration executable will only be run if the version 
	//rules match the current version of Inventor that is being registered
	VersionSupportRule[] m_supportedVersions = new VersionSupportRule[]
	{
		new VersionSupportRule(VersionSupportType.SupportedSoftwareVersionGreaterThan, "11..")
	};
	public VersionSupportRule[] SupportedVersions{ get{return m_supportedVersions;}}

	//.NET assemblies to register (which in this case is the "SimpleAddIn.dll" that this re-registration executable is associated with
    //the assemblies need to be specified with a relative path w.r.t this re-registration executable
    //the re-registration executable is normally meant to be used by the installer of the AddIn.
    //in this case, the "Installer" folder contains this sample to install and register the AddIn. 
    //during installation, the installer places the SimpleAddIn.dll and the Registration.exe 
    //(re-registration executable) in the same folder, hence no path needs to be specified.
	string[] m_dotNetAssemblies = new string[] 
	{
		"InvAddIn.dll"
	};

	//standard Dll's to register
	//if AddInDLL1.dll and AddInDLL2.dll are the dll's that need to be registered,
	//import the entry point for "DllRegisterServer" alias it and call it in the "Register" method below
	//[DllImport("AddInDLL1.dll", EntryPoint="DllRegisterServer")]
	//internal static extern void Register_AddInDLL1();

	//[DllImport("AddInDLL2.dll", EntryPoint="DllRegisterServer")]
	//internal static extern void Register_AddInDLL2();
	
	//import the entry point for "DllUnregisterServer" alias it and call it in the "Unregister" method below
	//[DllImport("AddInDLL1.dll", EntryPoint="DllUnregisterServer")]
	//internal static extern void Unregister_AddInDLL1();

	//[DllImport("AddInDLL2.dll", EntryPoint="DllUnregisterServer")]
	//internal static extern void Unregister_AddInDLL2();

	public void Register() 
	{
		//register .NET assemblies
		Utils.RegisterDotNetAssemblies(Path.GetDirectoryName(ExecutablePath), m_dotNetAssemblies, true, true);
		
		//register standard dll's, if any
		//Register_AddInDLL1();
		//Register_AddInDLL2();
	}		
	public void Unregister() 
	{
		//unregister .NET assemblies
		Utils.RegisterDotNetAssemblies(Path.GetDirectoryName(ExecutablePath), m_dotNetAssemblies, false, true);
		
		//unregister standard dll's, if any
		//Unregister_AddInDLL1();
		//Unregister_AddInDLL2();
	}	

	[STAThread]
	static public int Main(string[] args) 
	{
		return InventorRegistration.Utils.DoAddInRegistration(args, new SimpleAddInRegInfo());
	}
}